// Downloaded from https://repo.progsbase.com - Code Developed Using progsbase.

function CreateFraction(n, d){
  var f;

  f = {};
  f.n = n;
  f.d = d;

  return f;
}
function Copy(a){
  var f;

  f = {};
  f.n = a.n;
  f.d = a.d;

  return f;
}
function Assign(x, a){
  x.n = a.n;
  x.d = a.d;
}
function Add(a, b){
  var lcm, amul, bmul;

  lcm = LeastCommonMultiple(a.d, b.d);
  amul = lcm/a.d;
  bmul = lcm/b.d;

  a.n = amul*a.n + bmul*b.n;
  a.d = lcm;

  Simplify(a);
}
function Subtract(a, b){
  var lcm, amul, bmul;

  lcm = LeastCommonMultiple(a.d, b.d);
  amul = lcm/a.d;
  bmul = lcm/b.d;

  a.n = amul*a.n - bmul*b.n;
  a.d = lcm;

  Simplify(a);
}
function Multiply(a, b){
  var gcd;

  a.n = a.n*b.n;
  a.d = a.d*b.d;

  Simplify(a);
}
function Divide(a, b){
  a.n = a.n*b.d;
  a.d = a.d*b.n;

  Simplify(a);
}
function Simplify(a){
  var gcd;

  gcd = GreatestCommonDivisor(a.n, a.d);

  a.n = a.n/gcd;
  a.d = a.d/gcd;
}
function CreateFractionFromNumber(n, precision){
  return CreateFractionFromNumberFareyAlgoroithm(n, precision);
}
function CreateFractionFromNumberSimpleAlgorithm(n, precision){
  var f;

  f = {};

  f.n = Math.floor(n/precision);
  f.d = Math.floor(1/precision);

  Simplify(f);

  return f;
}
function FractionToDouble(f){
  return f.n/f.d;
}
function CreateFractionFromNumberFareyAlgoroithm(n, precision){
  var f, u, l, c, cPrevious;
  var negative, done;
  var w, cDistance, lDistance, uDistance;

  /* Factor out integtal part. */
  if(n < 0){
    n =  -n;
    negative = true;
  }else{
    negative = false;
  }

  if(n >= 1){
    w = Math.floor(n);
    n = n - w;
  }else{
    w = 0;
  }

  f = {};
  u = CreateFraction(1, 1);
  l = CreateFraction(0, 1);
  c = CreateFraction(0, 0);
  cPrevious = CreateFraction(0, 0);

  /* Core algorithm. */
  done = false;
  for(;  !done ; ){
    Assign(cPrevious, c);

    Assign(c, l);
    FareyAdd(c, u);

    if(c.n > 1/precision || c.d > 1/precision){
      done = true;
    }else if(FractionToDouble(c) <= n){
      Assign(l, c);
    }else{
      Assign(u, c);
    }
  }

  cDistance = Math.abs(FractionToDouble(cPrevious) - n);
  lDistance = Math.abs(FractionToDouble(l) - n);
  uDistance = Math.abs(FractionToDouble(u) - n);

  if(cDistance <= Math.min(lDistance, uDistance)){
    Assign(f, cPrevious);
  }else if(lDistance <= Math.min(cDistance, uDistance)){
    Assign(f, l);
  }else if(uDistance <= Math.min(cDistance, lDistance)){
    Assign(f, u);
  }

  delete(c);
  delete(u);
  delete(l);

  /* Adjust with integral part. */
  if(negative){
    f.n =  -f.n;
  }
  f.n = f.n + f.d*w;

  return f;
}
function FareyAdd(a, b){
  a.n = a.n + b.n;
  a.d = a.d + b.d;

  Simplify(a);
}
function testFractions(){
  var failures;
  var a, b;

  failures = CreateNumberReference(0);

  a = CreateFraction(1, 3);
  b = CreateFraction(1, 4);

  Add(a, b);

  AssertEquals(7, a.n, failures);
  AssertEquals(12, a.d, failures);

  a = CreateFraction(1, 3);
  b = CreateFraction(1, 4);

  Subtract(a, b);

  AssertEquals(1, a.n, failures);
  AssertEquals(12, a.d, failures);

  a = CreateFraction(1, 3);
  b = CreateFraction(1, 4);

  Multiply(a, b);

  AssertEquals(1, a.n, failures);
  AssertEquals(12, a.d, failures);

  a = CreateFraction(1, 3);
  b = CreateFraction(1, 3);

  Divide(a, b);

  AssertEquals(1, a.n, failures);
  AssertEquals(1, a.d, failures);

  a = CreateFractionFromNumber(1.25, 0.001);

  AssertEquals(5, a.n, failures);
  AssertEquals(4, a.d, failures);

  a = CreateFractionFromNumber(0.336944434029, 1/10000);

  AssertEquals(1213, a.n, failures);
  AssertEquals(3600, a.d, failures);

  return failures.numberValue;
}
function test(){
  var failures;

  failures = 0;

  failures = failures + testFractions();

  return failures;
}
function Negate(x){
  return  -x;
}
function Positive(x){
  return  +x;
}
function Factorial(x){
  var i, f;
  f = 1;

  for(i = 2; i <= x; i = i + 1){
    f = f*i;
  }

  return f;
}
function Round(x){
  return Math.floor(x + 0.5);
}
function BankersRound(x){
  var r;
  if(Absolute(x - Truncate(x)) == 0.5){
    if( !DivisibleBy(Round(x), 2) ){
      r = Round(x) - 1;
    }else{
      r = Round(x);
    }
  }else{
    r = Round(x);
  }

  return r;
}
function Ceil(x){
  return Math.ceil(x);
}
function Floor(x){
  return Math.floor(x);
}
function Truncate(x){
  var t;
  if(x >= 0){
    t = Math.floor(x);
  }else{
    t = Math.ceil(x);
  }

  return t;
}
function Absolute(x){
  return Math.abs(x);
}
function Logarithm(x){
  return Math.log10(x);
}
function NaturalLogarithm(x){
  return Math.log(x);
}
function Sin(x){
  return Math.sin(x);
}
function Cos(x){
  return Math.cos(x);
}
function Tan(x){
  return Math.tan(x);
}
function Asin(x){
  return Math.asin(x);
}
function Acos(x){
  return Math.acos(x);
}
function Atan(x){
  return Math.atan(x);
}
function Atan2(y, x){
  var a;
  a = 0;

  if(x > 0){
    a = Atan(y/x);
  }else if(x < 0 && y >= 0){
    a = Atan(y/x) + Math.PI;
  }else if(x < 0 && y < 0){
    a = Atan(y/x) - Math.PI;
  }else if(x == 0 && y > 0){
    a = Math.PI/2;
  }else if(x == 0 && y < 0){
    a =  -Math.PI/2;
  }

  return a;
}
function Squareroot(x){
  return Math.sqrt(x);
}
function Exp(x){
  return Math.exp(x);
}
function DivisibleBy(a, b){
  return ((a%b) == 0);
}
function Combinations(n, k){
  return Factorial(n)/(Factorial(n - k)*Factorial(k));
}
function EpsilonCompareApproximateDigits(a, b, digits){
  var ad, bd, d, epsilon;
  var ret;
  if(a < 0 && b < 0 || a > 0 && b > 0){
    if(a < 0 && b < 0){
      a =  -a;
      b =  -b;
    }
    ad = Math.log10(a);
    bd = Math.log10(b);
    d = Math.max(ad, bd);
    epsilon = Math.pow(10, d - digits);
    ret = Math.abs(a - b) > epsilon;
  }else{
    ret = false;
  }

  return ret;
}
function EpsilonCompare(a, b, epsilon){
  return Math.abs(a - b) < epsilon;
}
function GreatestCommonDivisor(a, b){
  var t;
  for(; b != 0; ){
    t = b;
    b = a%b;
    a = t;
  }

  return a;
}
function IsInteger(a){
  return (a - Math.floor(a)) == 0;
}
function GreatestCommonDivisorWithCheck(a, b, gcdReference){
  var success;
  var gcd;
  if(IsInteger(a) && IsInteger(b)){
    gcd = GreatestCommonDivisor(a, b);
    gcdReference.numberValue = gcd;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function LeastCommonMultiple(a, b){
  var lcm;
  if(a > 0 && b > 0){
    lcm = Math.abs(a*b)/GreatestCommonDivisor(a, b);
  }else{
    lcm = 0;
  }

  return lcm;
}
function Sign(a){
  var s;
  if(a > 0){
    s = 1;
  }else if(a < 0){
    s =  -1;
  }else{
    s = 0;
  }

  return s;
}
function Max(a, b){
  return Math.max(a, b);
}
function Min(a, b){
  return Math.min(a, b);
}
function Power(a, b){
  return Math.pow(a, b);
}
function AssertFalse(b, failures){
  if(b){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertTrue(b, failures){
  if( !b ){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertEquals(a, b, failures){
  if(a != b){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertBooleansEqual(a, b, failures){
  if(a != b){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertCharactersEqual(a, b, failures){
  if(a != b){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertStringEquals(a, b, failures){
  if( !StringsEqual(a, b) ){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertNumberArraysEqual(a, b, failures){
  var i;
  if(a.length == b.length){
    for(i = 0; i < a.length; i = i + 1){
      AssertEquals(a[i], b[i], failures);
    }
  }else{
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertBooleanArraysEqual(a, b, failures){
  var i;
  if(a.length == b.length){
    for(i = 0; i < a.length; i = i + 1){
      AssertBooleansEqual(a[i], b[i], failures);
    }
  }else{
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertStringArraysEqual(a, b, failures){
  var i;
  if(a.length == b.length){
    for(i = 0; i < a.length; i = i + 1){
      AssertStringEquals(a[i].string, b[i].string, failures);
    }
  }else{
    failures.numberValue = failures.numberValue + 1;
  }
}
function CreateBooleanReference(value){
  var ref;
  ref = {};
  ref.booleanValue = value;

  return ref;
}
function CreateBooleanArrayReference(value){
  var ref;
  ref = {};
  ref.booleanArray = value;

  return ref;
}
function CreateBooleanArrayReferenceLengthValue(lengthx, value){
  var ref;
  var i;
  ref = {};
  ref.booleanArray = [];
  ref.booleanArray.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.booleanArray[i] = value;
  }

  return ref;
}
function FreeBooleanArrayReference(booleanArrayReference){
  delete(booleanArrayReference.booleanArray);
  delete(booleanArrayReference);
}
function CreateCharacterReference(value){
  var ref;
  ref = {};
  ref.characterValue = value;

  return ref;
}
function CreateNumberReference(value){
  var ref;
  ref = {};
  ref.numberValue = value;

  return ref;
}
function CreateNumberArrayReference(value){
  var ref;
  ref = {};
  ref.numberArray = value;

  return ref;
}
function CreateNumberArrayReferenceLengthValue(lengthx, value){
  var ref;
  var i;
  ref = {};
  ref.numberArray = [];
  ref.numberArray.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.numberArray[i] = value;
  }

  return ref;
}
function FreeNumberArrayReference(numberArrayReference){
  delete(numberArrayReference.numberArray);
  delete(numberArrayReference);
}
function CreateStringReference(value){
  var ref;
  ref = {};
  ref.string = value;

  return ref;
}
function CreateStringReferenceLengthValue(lengthx, value){
  var ref;
  var i;
  ref = {};
  ref.string = [];
  ref.string.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.string[i] = value;
  }

  return ref;
}
function FreeStringReference(stringReference){
  delete(stringReference.string);
  delete(stringReference);
}
function CreateStringArrayReference(strings){
  var ref;
  ref = {};
  ref.stringArray = strings;

  return ref;
}
function CreateStringArrayReferenceLengthValue(lengthx, value){
  var ref;
  var i;
  ref = {};
  ref.stringArray = [];
  ref.stringArray.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.stringArray[i] = CreateStringReference(value);
  }

  return ref;
}
function FreeStringArrayReference(stringArrayReference){
  var i;
  for(i = 0; i < stringArrayReference.stringArray.length; i = i + 1){
    delete(stringArrayReference.stringArray[i]);
  }
  delete(stringArrayReference.stringArray);
  delete(stringArrayReference);
}
function StringToNumberArray(string){
  var i;
  var array;
  array = [];
  array.length = string.length;

  for(i = 0; i < string.length; i = i + 1){
    array[i] = string[i].charCodeAt(0);
  }
  return array;
}
function NumberArrayToString(array){
  var i;
  var string;
  string = [];
  string.length = array.length;

  for(i = 0; i < array.length; i = i + 1){
    string[i] = String.fromCharCode(array[i]);
  }
  return string;
}
function NumberArraysEqual(a, b){
  var equal;
  var i;
  equal = true;
  if(a.length == b.length){
    for(i = 0; i < a.length && equal; i = i + 1){
      if(a[i] != b[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function BooleanArraysEqual(a, b){
  var equal;
  var i;
  equal = true;
  if(a.length == b.length){
    for(i = 0; i < a.length && equal; i = i + 1){
      if(a[i] != b[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function StringsEqual(a, b){
  var equal;
  var i;
  equal = true;
  if(a.length == b.length){
    for(i = 0; i < a.length && equal; i = i + 1){
      if(a[i] != b[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function FillNumberArray(a, value){
  var i;
  for(i = 0; i < a.length; i = i + 1){
    a[i] = value;
  }
}
function FillString(a, value){
  var i;
  for(i = 0; i < a.length; i = i + 1){
    a[i] = value;
  }
}
function FillBooleanArray(a, value){
  var i;
  for(i = 0; i < a.length; i = i + 1){
    a[i] = value;
  }
}
function FillNumberArrayInterval(a, value, from, to){
  var i;
  var success;
  if(from >= 0 && from < a.length && to >= 0 && to < a.length){
    for(i = from; i < to; i = i + 1){
      a[i] = value;
    }
    success = true;
  }else{
    success = false;
  }

  return success;
}
function FillBooleanArrayInterval(a, value, from, to){
  var i;
  var success;
  if(from >= 0 && from < a.length && to >= 0 && to < a.length){
    for(i = Math.max(from, 0); i < Math.min(to, a.length); i = i + 1){
      a[i] = value;
    }
    success = true;
  }else{
    success = false;
  }

  return success;
}
function FillStringInterval(a, value, from, to){
  var i;
  var success;
  if(from >= 0 && from < a.length && to >= 0 && to < a.length){
    for(i = Math.max(from, 0); i < Math.min(to, a.length); i = i + 1){
      a[i] = value;
    }
    success = true;
  }else{
    success = false;
  }

  return success;
}
function CopyNumberArray(a){
  var i;
  var n;
  n = [];
  n.length = a.length;

  for(i = 0; i < a.length; i = i + 1){
    n[i] = a[i];
  }

  return n;
}
function CopyBooleanArray(a){
  var i;
  var n;
  n = [];
  n.length = a.length;

  for(i = 0; i < a.length; i = i + 1){
    n[i] = a[i];
  }

  return n;
}
function CopyString(a){
  var i;
  var n;
  n = [];
  n.length = a.length;

  for(i = 0; i < a.length; i = i + 1){
    n[i] = a[i];
  }

  return n;
}
function CopyNumberArrayRange(a, from, to, copyReference){
  var i, lengthx;
  var n;
  var success;
  if(from >= 0 && from < a.length && to >= 0 && to < a.length && from <= to){
    lengthx = to - from;
    n = [];
    n.length = lengthx;
    for(i = 0; i < lengthx; i = i + 1){
      n[i] = a[from + i];
    }
    copyReference.numberArray = n;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function CopyBooleanArrayRange(a, from, to, copyReference){
  var i, lengthx;
  var n;
  var success;
  if(from >= 0 && from < a.length && to >= 0 && to < a.length && from <= to){
    lengthx = to - from;
    n = [];
    n.length = lengthx;
    for(i = 0; i < lengthx; i = i + 1){
      n[i] = a[from + i];
    }
    copyReference.booleanArray = n;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function CopyStringRange(a, from, to, copyReference){
  var i, lengthx;
  var n;
  var success;
  if(from >= 0 && from < a.length && to >= 0 && to < a.length && from <= to){
    lengthx = to - from;
    n = [];
    n.length = lengthx;
    for(i = 0; i < lengthx; i = i + 1){
      n[i] = a[from + i];
    }
    copyReference.string = n;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function IsLastElement(lengthx, index){
  return index + 1 == lengthx;
}
function CreateNumberArray(lengthx, value){
  var array;
  array = [];
  array.length = lengthx;
  FillNumberArray(array, value);

  return array;
}
function CreateBooleanArray(lengthx, value){
  var array;
  array = [];
  array.length = lengthx;
  FillBooleanArray(array, value);

  return array;
}
function CreateString(lengthx, value){
  var array;
  array = [];
  array.length = lengthx;
  FillString(array, value);

  return array;
}


module.exports = {
	CreateFraction: CreateFraction,
	Copy: Copy,
	Assign: Assign,
	Add: Add,
	Subtract: Subtract,
	Multiply: Multiply,
	Divide: Divide,
	Simplify: Simplify,
	CreateFractionFromNumber: CreateFractionFromNumber,
	CreateFractionFromNumberSimpleAlgorithm: CreateFractionFromNumberSimpleAlgorithm,
	FractionToDouble: FractionToDouble,
	CreateFractionFromNumberFareyAlgoroithm: CreateFractionFromNumberFareyAlgoroithm,
	FareyAdd: FareyAdd,
	Negate: Negate,
	Positive: Positive,
	Factorial: Factorial,
	Round: Round,
	BankersRound: BankersRound,
	Ceil: Ceil,
	Floor: Floor,
	Truncate: Truncate,
	Absolute: Absolute,
	Logarithm: Logarithm,
	NaturalLogarithm: NaturalLogarithm,
	Sin: Sin,
	Cos: Cos,
	Tan: Tan,
	Asin: Asin,
	Acos: Acos,
	Atan: Atan,
	Atan2: Atan2,
	Squareroot: Squareroot,
	Exp: Exp,
	DivisibleBy: DivisibleBy,
	Combinations: Combinations,
	EpsilonCompareApproximateDigits: EpsilonCompareApproximateDigits,
	EpsilonCompare: EpsilonCompare,
	GreatestCommonDivisor: GreatestCommonDivisor,
	IsInteger: IsInteger,
	GreatestCommonDivisorWithCheck: GreatestCommonDivisorWithCheck,
	LeastCommonMultiple: LeastCommonMultiple,
	Sign: Sign,
	Max: Max,
	Min: Min,
	Power: Power,
	AssertFalse: AssertFalse,
	AssertTrue: AssertTrue,
	AssertEquals: AssertEquals,
	AssertBooleansEqual: AssertBooleansEqual,
	AssertCharactersEqual: AssertCharactersEqual,
	AssertStringEquals: AssertStringEquals,
	AssertNumberArraysEqual: AssertNumberArraysEqual,
	AssertBooleanArraysEqual: AssertBooleanArraysEqual,
	AssertStringArraysEqual: AssertStringArraysEqual,
	CreateBooleanReference: CreateBooleanReference,
	CreateBooleanArrayReference: CreateBooleanArrayReference,
	CreateBooleanArrayReferenceLengthValue: CreateBooleanArrayReferenceLengthValue,
	FreeBooleanArrayReference: FreeBooleanArrayReference,
	CreateCharacterReference: CreateCharacterReference,
	CreateNumberReference: CreateNumberReference,
	CreateNumberArrayReference: CreateNumberArrayReference,
	CreateNumberArrayReferenceLengthValue: CreateNumberArrayReferenceLengthValue,
	FreeNumberArrayReference: FreeNumberArrayReference,
	CreateStringReference: CreateStringReference,
	CreateStringReferenceLengthValue: CreateStringReferenceLengthValue,
	FreeStringReference: FreeStringReference,
	CreateStringArrayReference: CreateStringArrayReference,
	CreateStringArrayReferenceLengthValue: CreateStringArrayReferenceLengthValue,
	FreeStringArrayReference: FreeStringArrayReference,
	StringToNumberArray: StringToNumberArray,
	NumberArrayToString: NumberArrayToString,
	NumberArraysEqual: NumberArraysEqual,
	BooleanArraysEqual: BooleanArraysEqual,
	StringsEqual: StringsEqual,
	FillNumberArray: FillNumberArray,
	FillString: FillString,
	FillBooleanArray: FillBooleanArray,
	FillNumberArrayInterval: FillNumberArrayInterval,
	FillBooleanArrayInterval: FillBooleanArrayInterval,
	FillStringInterval: FillStringInterval,
	CopyNumberArray: CopyNumberArray,
	CopyBooleanArray: CopyBooleanArray,
	CopyString: CopyString,
	CopyNumberArrayRange: CopyNumberArrayRange,
	CopyBooleanArrayRange: CopyBooleanArrayRange,
	CopyStringRange: CopyStringRange,
	IsLastElement: IsLastElement,
	CreateNumberArray: CreateNumberArray,
	CreateBooleanArray: CreateBooleanArray,
	CreateString: CreateString,
	test: test
};